#include "stdafx.h"

#include "IOBase.h"

/*******************************
********************************
***
** CModule
***
********************************
*******************************/
CModule::CModule(BYTE *pModuleDef)
{
	XCount		= 0;
	YCount		= 0;
	WXCount		= 0;
	WYCount		= 0;
	DWXCount		= 0;
	DWYCount		= 0;

	memset(X, 0, sizeof(X));
	memset(Y, 0, sizeof(Y));
	memset(WX, 0, sizeof(WX));
	memset(WY, 0, sizeof(WY));
	memset(DWX, 0, sizeof(DWX));
	memset(DWY, 0, sizeof(DWY));

	switch(pModuleDef[0])
		{
		case 1:
			ModuleID = pModuleDef[1];
			XCount = pModuleDef[2];
			YCount = pModuleDef[3];
			WXCount = pModuleDef[4];
			WYCount = pModuleDef[5];
			break;

		case 3:
			ModuleID = pModuleDef[1];
			XCount = pModuleDef[2];
			YCount = pModuleDef[3];
			DWXCount = pModuleDef[4];
			DWYCount = pModuleDef[5];
			break;

		case 5:
			ModuleID = pModuleDef[1];
			XCount = pModuleDef[2];
			YCount = pModuleDef[3];
			WXCount = pModuleDef[4];
			WYCount = pModuleDef[5];
			DWXCount = pModuleDef[6];
			DWYCount = pModuleDef[7];
			break;

		case 7:
			{
			ModuleID = pModuleDef[1];
			for(int idx = 0; idx < pModuleDef[2]; idx++)
				{
				BYTE Type = pModuleDef[3 + idx*2];
				BYTE Count = pModuleDef[4 + idx*2];
				switch(Type)
					{
					case DF_BIT_IN:		XCount = Count;	break;
					case DF_BIT_OUT:		YCount = Count;	break;
					case DF_WORD_IN:		WXCount = Count;	break;
					case DF_WORD_OUT:		WYCount = Count;	break;
					case DF_DWORD_IN:		DWXCount = Count;	break;
					case DF_DWORD_OUT:	DWYCount = Count;	break;
					}
				}
			}
			break;

		case 255:
			ModuleID = pModuleDef[1];
			break;
		}
}

BOOL CModule::IsSupported(BYTE *pModuleDef)
{
	switch(pModuleDef[0])
		{
		case 1:
		case 3:
		case 5:
		case 7:
		case 255:
			return TRUE;

		default:
			return FALSE;
		}
}

int CModule::WriteModuleState(BYTE *pBuff, BOOL UseNewMode)
	{
	int Offset = 0;
	if(YCount)
		{
		int ByteCount = (YCount+7)/8;
		
		if(UseNewMode)
			{
			pBuff[Offset++] = 0x04;
			pBuff[Offset++] = ByteCount;
			}
		else
			{
			pBuff[Offset++] = 0x40 + ByteCount;
			}
		
		for(int idx = 0; idx < ByteCount; idx++)
			pBuff[Offset++] = Y[idx];
		}

	if(WYCount)
		{
		if(UseNewMode)
			{
			pBuff[Offset++] = 0x06;
			pBuff[Offset++] = WYCount * 2;
			}
		else
			{
			pBuff[Offset++] = 0x60 + WYCount;
			}

		for(int idx = 0; idx < WYCount; idx++)
			{
			*(WORD *)(pBuff + Offset) = WY[idx];
			Offset += 2;
			}
		}

	if(DWYCount)
		{
		if(UseNewMode)
			{
			pBuff[Offset++] = 0x09;
			pBuff[Offset++] = DWYCount * 4;
			}
		else
			{
			pBuff[Offset++] = 0x90 + DWYCount;
			}

		for(int idx = 0; idx < DWYCount; idx++)
			{
			*(DWORD *)(pBuff + Offset) = DWY[idx];
			Offset += 4;
			}
		}

	return Offset;
	}

/*******************************
********************************
***
** CEbcIO
***
********************************
*******************************/
CEbcIO::CEbcIO(HEIDevice *ipDevice, BOOL iUseNewMode)
{
	pDevice = ipDevice;
	UseNewMode = iUseNewMode;
	Error = 0;
	MaxCount = 0;

	memset(aModules, 0, sizeof(aModules));
	IndexOutputs = FALSE;

	/*
	** Read base contents
	*/
	int ModuleCount = 0;
	BYTE BaseDef[1000];
	WORD BaseDefSize = sizeof(BaseDef);

	// Signal new mode
	if(UseNewMode)
		{
		BaseDef[0] = 0xb1;
		BaseDef[1] = 0;
		}

	if(!HEIReadBaseDef(pDevice, BaseDef, &BaseDefSize))
		{
		BOOL Done = FALSE;
		int Slot = -1;
		int Base = 0;
		BOOL NewStyle = FALSE;

		// Determine slot contents
		for(int idx = 0; idx < BaseDefSize && !Done;)
			{
			BYTE FunCode = NewStyle ? BaseDef[idx++] : ((BaseDef[idx] & 0xF0) >> 4);
			BYTE FunParm = NewStyle ? BaseDef[idx++] : (BaseDef[idx++] & 0x0F);

			switch(FunCode)
				{
				// Select Base
				case 0x07:
					Base = FunParm;
					break;

				// Select Slot
				case 0x00:
					Slot = FunParm;
					break;

				// Module def
				case 0x01:
					if(CModule::IsSupported(&BaseDef[idx]))
						{
						aModules[Slot] = new CModule(&BaseDef[idx]);
						if(Slot+1 > MaxCount) MaxCount = Slot+1;
						}

					idx += 2 + FunParm;
					break;

				// Select new style
				case 0x0B:
					idx += FunParm;
					NewStyle = TRUE;
					break;

				// Finished
				case 0xF0:
					Done = TRUE;
					break;
				}
			}
		}


	/*
	** Calc IO offsets
	*/
	memset(aXInSlot, 0, sizeof(aXInSlot));
	memset(aYInSlot, 0, sizeof(aYInSlot));
	memset(aWXInSlot, 0, sizeof(aWXInSlot));
	memset(aWYInSlot, 0, sizeof(aWYInSlot));
	memset(aDWXInSlot, 0, sizeof(aDWXInSlot));
	memset(aDWYInSlot, 0, sizeof(aDWYInSlot));
	WORD CurX = 0, CurY = 0, CurWX = 0, CurWY = 0, CurDWX = 0, CurDWY = 0;
	for(int idx = 0; idx < MAX_MODULE_COUNT; idx++)
		{
		CModule *pMod = aModules[idx];
		if(pMod)
			{
			if(pMod->XCount)
				{
				CurX += pMod->XCount;
				aXInSlot[idx] = CurX;
				}

			if(pMod->YCount)
				{
				CurY += pMod->YCount;
				aYInSlot[idx] = CurY;
				}

			if(pMod->WXCount)
				{
				CurWX += pMod->WXCount;
				aWXInSlot[idx] = CurWX;
				}

			if(pMod->WYCount)
				{
				CurWY += pMod->WYCount;
				aWYInSlot[idx] = CurWY;
				}

			if(pMod->DWXCount)
				{
				CurDWX += pMod->DWXCount;
				aDWXInSlot[idx] = CurDWX;
				}

			if(pMod->DWYCount)
				{
				CurDWY += pMod->DWYCount;
				aDWYInSlot[idx] = CurDWY;
				}
			}
		}

	/*
	** Initial read
	*/
	ReadIO();
}

CEbcIO::~CEbcIO()
{
	for(int idx = 0; idx < MAX_MODULE_COUNT; idx++)
		{
		delete aModules[idx];
		aModules[idx] = NULL;
		}
}

void CEbcIO::ParseInputPacket(int Length)
{
	BOOL Done = FALSE;
	int Slot = -1;
	int Base = 0;
	BOOL NewStyle = FALSE;
	CModule *pMod = aModules[0];

	for(int idx = 0; idx < Length && !Done;)
		{
		BYTE FunCode = NewStyle ? InBuff[idx++] : ((InBuff[idx] & 0xF0) >> 4);
		BYTE FunParm = NewStyle ? InBuff[idx++] : (InBuff[idx++] & 0x0F);

		switch(FunCode)
			{
			case 0x07:
				Base = FunParm;
				break;

			case 0x00:
				Slot = FunParm;
				pMod = aModules[Slot];
				break;

			case 0x01:
				idx += 2 + FunParm;
				break;

			case 0x02:
				idx += FunParm;
				break;

			// Discrete in
			case 0x03:
				if(pMod)
					pMod->SetXState(InBuff+idx, FunParm);

				idx += FunParm;
				break;

			// Discrete out
			case 0x04:
				if(pMod)
					pMod->SetYState(InBuff+idx, FunParm);

				idx += FunParm;
				break;

			// Word in
			case 0x05:
				if(pMod)
					pMod->SetWXState((WORD *)(InBuff+idx), NewStyle ? FunParm : FunParm * 2);

				idx += FunParm;
				break;

			// Word out
			case 0x06:
				if(pMod)
					pMod->SetWYState((WORD *)(InBuff+idx), NewStyle ? FunParm : FunParm * 2);

				idx += FunParm;
				break;

			// Dword in
			case 0x08:
				if(pMod)
					pMod->SetDWXState((DWORD *)(InBuff+idx), NewStyle ? FunParm : FunParm * 4);

				idx += FunParm;
				break;

			// Dword out
			case 0x09:
				if(pMod)
					pMod->SetDWYState((DWORD *)(InBuff+idx), NewStyle ? FunParm : FunParm * 4);

				idx += FunParm;
				break;
				
			case 0x0B:
				idx += FunParm;
				NewStyle = TRUE;
				break;

			case 0x0A:
			case 0x0C:
			case 0x0D:
				idx += FunParm;
				break;

			case 0x0E:
				idx += 2 + FunParm;
				break;

			case 0x0F:
				Done = TRUE;
				break;
			}
		}
}


int CEbcIO::PackOutputPacket()
	{
	int Offset = 0;

	// Signal new mode
	if(UseNewMode)
		{
		OutBuff[0] = 0xb1;
		OutBuff[1] = 0;
		Offset = 2;
		}

	for(int idx = 0; idx < MAX_MODULE_COUNT; idx++)
		{
		if(aModules[idx] && aModules[idx]->IsOut())
			{
			if(UseNewMode)
				{
				OutBuff[Offset++] = 0;
				OutBuff[Offset++] = idx;
				}
			else
				{
				OutBuff[Offset++] = idx;
				}

			Offset += aModules[idx]->WriteModuleState(OutBuff+Offset, UseNewMode);
			}
		}

	OutBuff[Offset++] = 0xFF;
	return Offset;
	}

BOOL CEbcIO::ReadIO()
	{
	WORD Length = sizeof(InBuff);

	// Signal new mode
	if(UseNewMode)
		{
		InBuff[0] = 0xb1;
		InBuff[1] = 0;
		}

	Error = HEIReadIO(pDevice, InBuff, &Length);
	if(DataGood())
		{
		ParseInputPacket(Length);
		return FALSE;
		}
	else
		return TRUE;
	}

BOOL CEbcIO::WriteIO()
	{
	WORD Length = PackOutputPacket();
	WORD InLength = sizeof(InBuff);

	Error = HEIWriteIO(pDevice, OutBuff, Length, InBuff, &InLength);
	if(DataGood())
		{
		ParseInputPacket(InLength);
		return FALSE;
		}
	else
		return TRUE;
	}

BOOL CEbcIO::GetDI(short Slot, short Point)
	{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->XCount)
				{
				return aModules[Slot]->GetX(Point);
				}
			}
		}

	return -1;
	}

BOOL CEbcIO::GetDO(short Slot, short Point)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->YCount)
				{
				return aModules[Slot]->GetY(Point);
				}
			}
		}

	return -1;
}

BOOL CEbcIO::SetDO(short Slot, short Point, BOOL NewVal)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->YCount)
				{
				aModules[Slot]->SetY(Point, !!NewVal);
				return 0;
				}
			}
		}

	return -1;
}

WORD CEbcIO::GetWI(short Slot, short Point)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WXCount)
				{
				return aModules[Slot]->WX[Point];
				}
			}
		}

	return -1;
}

WORD CEbcIO::GetWO(short Slot, short Point)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WYCount)
				{
				return aModules[Slot]->WY[Point];
				}
			}
		}

	return -1;
}

BOOL CEbcIO::SetWO(short Slot, short Point, WORD NewVal)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->WYCount)
				{
				aModules[Slot]->WY[Point] = NewVal;
				return 0;
				}
			}
		}

	return -1;
}

DWORD CEbcIO::GetDWI(short Slot, short Point)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWXCount)
				{
				return aModules[Slot]->DWX[Point];
				}
			}
		}

	return -1;
}

DWORD CEbcIO::GetDWO(short Slot, short Point)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWYCount)
				{
				return aModules[Slot]->DWY[Point];
				}
			}
		}

	return -1;
}

BOOL CEbcIO::SetDWO(short Slot, short Point, DWORD NewVal)
{
	if(Slot < MAX_MODULE_COUNT)
		{
		if(aModules[Slot])
			{
			if(Point < aModules[Slot]->DWYCount)
				{
				aModules[Slot]->DWY[Point] = NewVal;
				return 0;
				}
			}
		}

	return -1;
}

BOOL CEbcIO::GetX(short Point)
{
	WORD LastX = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aXInSlot[Slot])
			{
			if(Point >= LastX && Point < aXInSlot[Slot])
				return aModules[Slot]->GetX(Point - LastX);

			LastX += aXInSlot[Slot];
			}
		}

	return -1;
}

BOOL CEbcIO::GetY(short Point)
{
	WORD LastY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aYInSlot[Slot])
			{
			if(Point >= LastY && Point < aYInSlot[Slot])
				return aModules[Slot]->GetY(Point - LastY);

			LastY += aYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CEbcIO::SetY(short Point, BOOL NewVal)
{
	WORD LastY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aYInSlot[Slot])
			{
			if(Point >= LastY && Point < aYInSlot[Slot])
				{
				aModules[Slot]->SetY(Point - LastY, NewVal);
				return 0;
				}
	
			LastY += aYInSlot[Slot];
			}
		}

	return -1;
}

WORD CEbcIO::GetWX(short Point)
{
	WORD LastWX = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aWXInSlot[Slot])
			{
			if(Point >= LastWX && Point < aWXInSlot[Slot])
				return aModules[Slot]->WX[Point-LastWX];

			LastWX += aWXInSlot[Slot];
			}
		}

	return -1;
}

WORD CEbcIO::GetWY(short Point)
{
	WORD LastWY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aWYInSlot[Slot])
			{
			if(Point >= LastWY && Point < aWYInSlot[Slot])
				return aModules[Slot]->WY[Point-LastWY];

			LastWY += aWYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CEbcIO::SetWY(short Point, WORD NewVal)
{
	WORD LastWY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aWYInSlot[Slot])
			{
			if(Point >= LastWY && Point < aWYInSlot[Slot])
				{
				aModules[Slot]->WY[Point-LastWY] = NewVal;
				return 0;
				}

			LastWY += aWYInSlot[Slot];
			}
		}

	return -1;
}

DWORD CEbcIO::GetDWX(short Point)
{
	WORD LastDWX = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aDWXInSlot[Slot])
			{
			if(Point >= LastDWX && Point < aDWXInSlot[Slot])
				return aModules[Slot]->DWX[Point-LastDWX];

			LastDWX += aDWXInSlot[Slot];
			}
		}

	return -1;
}

DWORD CEbcIO::GetDWY(short Point)
{
	WORD LastDWY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aDWYInSlot[Slot])
			{
			if(Point >= LastDWY && Point < aDWYInSlot[Slot])
				return aModules[Slot]->DWY[Point-LastDWY];

			LastDWY += aDWYInSlot[Slot];
			}
		}

	return -1;
}

BOOL CEbcIO::SetDWY(short Point, DWORD NewVal)
{
	WORD LastDWY = 0;
	for(int Slot = 0; Slot < MAX_MODULE_COUNT; Slot++)
		{
		if(aDWYInSlot[Slot])
			{
			if(Point >= LastDWY && Point < aDWYInSlot[Slot])
				{
				aModules[Slot]->DWY[Point-LastDWY] = NewVal;
				return 0;
				}

			LastDWY += aDWYInSlot[Slot];
			}
		}

	return -1;
}

