#ifndef __IOBASE_H
#define __IOBASE_H

#include <string.h>

#define MAX_MODULE_COUNT	32

class CModule
	{
	public:
		CModule(BYTE *pModuleDef);

		static CModule::IsSupported(BYTE *pModuleDef);

		int WriteModuleState(BYTE *pBuff, BOOL UseNewMode);

		void SetXState(void *pX, int Len){ memcpy(X, pX, Len); }
		void SetYState(void *pY, int Len){ memcpy(Y, pY, Len); }
		void SetWXState(void *pWX, int Len){ memcpy(WX, pWX, Len); }
		void SetWYState(void *pWY, int Len){ memcpy(WY, pWY, Len); }
		void SetDWXState(void *pDWX, int Len){ memcpy(DWX, pDWX, Len); }
		void SetDWYState(void *pDWY, int Len){ memcpy(DWY, pDWY, Len); }
		
		BOOL IsIn(){ return XCount || WXCount || DWXCount; }
		BOOL IsOut(){ return YCount || WYCount || DWYCount; }
		BOOL IsDiscrete(){ return XCount || YCount; }
		BOOL IsAnalog(){ return WXCount || WYCount; }
		BOOL IsDouble(){ return DWXCount || DWYCount; }
		BOOL GetX(int BitNum){ return !!(X[BitNum/8] & (0x01<<(BitNum%8))); }
		BOOL GetY(int BitNum){ return !!(Y[BitNum/8] & (0x01<<(BitNum%8))); }
		void SetY(int BitNum, BOOL Val){ Y[BitNum/8] = (Y[BitNum/8] & ~(0x01<<(BitNum%8))) | (Val ? (0x01<<(BitNum%8)) : 0); }

		BYTE ModuleID;
		BYTE XCount;
		BYTE YCount;
		BYTE WXCount;
		BYTE WYCount;
		BYTE DWXCount;
		BYTE DWYCount;

		BYTE X[8];
		BYTE Y[8];
		WORD WX[32];
		WORD WY[32];
		DWORD DWX[32];
		DWORD DWY[32];
	};

class CEbcIO
	{
	public:
		CEbcIO(HEIDevice *pDevice, BOOL iUseNewMode = FALSE);
		~CEbcIO();
		
		// Read IO from device
		BOOL ReadIO();
		
		// Write IO to device. This also reads.
		BOOL WriteIO();

		// Get/set double words by slot:point
		DWORD GetDWI(short Slot, short Point);
		DWORD GetDWO(short Slot, short Point);
		BOOL SetDWO(short Slot, short Point, DWORD NewVal);
		
		// Get/set words by slot:point
		WORD GetWI(short Slot, short Point);
		WORD GetWO(short Slot, short Point);
		BOOL SetWO(short Slot, short Point, WORD NewVal);

		// Get/set bits by slot:point
		BOOL GetDI(short Slot, short Point);
		BOOL GetDO(short Slot, short Point);
		BOOL SetDO(short Slot, short Point, BOOL NewVal);

		// Get/set bit by PLC style point
		BOOL GetX(short Point);
		BOOL GetY(short Point);
		BOOL SetY(short Point, BOOL NewVal);

		// Get/set words by PLC style point
		WORD GetWX(short Point);
		WORD GetWY(short Point);
		BOOL SetWY(short Point, WORD NewVal);

		// Get/set dwords by PLC style point
		DWORD GetDWX(short Point);
		DWORD GetDWY(short Point);
		BOOL SetDWY(short Point, DWORD NewVal);

		int GetModuleCount(){ return MaxCount; }
		CModule *GetModule(int ModuleNum){ return aModules[ModuleNum]; }

	private:
		void ParseInputPacket(int Length);
		int PackOutputPacket();
		BOOL DataGood(){ return !((Error & 0x8000) && (Error & 0x0FFF)); }

		CModule *aModules[MAX_MODULE_COUNT];
		BYTE InBuff[1000];
		BYTE OutBuff[1000];
		BOOL IndexOutputs;
		HEIDevice *pDevice;
		WORD aXInSlot[MAX_MODULE_COUNT];
		WORD aYInSlot[MAX_MODULE_COUNT];
		WORD aWXInSlot[MAX_MODULE_COUNT];
		WORD aWYInSlot[MAX_MODULE_COUNT];
		WORD aDWXInSlot[MAX_MODULE_COUNT];
		WORD aDWYInSlot[MAX_MODULE_COUNT];
		int Error;
		BOOL UseNewMode;
		int MaxCount;
	};

#endif
